﻿using System.Text.Json;
using Connected.Configuration;
using Connected.Events;
using Connected.Net;
using Connected.Startup;
using Microsoft.AspNetCore.Components.WebAssembly.Hosting;
using Microsoft.Extensions.DependencyInjection;

namespace Connected;

public class ClientStartup : IStartup
{
	public static IServiceProvider Services { get; private set; } = default!;

	public async Task Configure(WebAssemblyHost host)
	{
		Services = host.Services;
		
		var events = host.Services.GetService<EventConnection>();

		if (events is null)
			throw new NullReferenceException(nameof(EventConnection));

		Console.WriteLine(JsonSerializer.Serialize(host.Configuration));

		await events.Initialize();
		await events.Connect();

		await Task.CompletedTask;
	}

	public async Task ConfigureServices(IServiceCollection services)
	{
		services.AddSingleton<EventConnection>();
		services.AddSingleton(typeof(IEventService), typeof(EventService));
		services.AddSingleton(typeof(IRemoteProviderService), typeof(RemoteProviderService));
		services.AddSingleton(typeof(IConfigurationService), typeof(ConfigurationService));

		await Task.CompletedTask;
	}
}
