﻿using Connected.Configuration;
using Microsoft.Extensions.Configuration;

namespace Connected.Net;

/// <summary>
/// This class should obtain all endpoints if the environment uses scalable setup.
/// </summary>
internal class RemoteProviderService : IRemoteProviderService
{
	public RemoteProviderService(IConfiguration configuration)
	{
		Configuration = configuration;
		Balancer = new();

		Initialize();
	}

	private RoundRobin Balancer { get; }
	private IConfiguration Configuration { get; }

	public string ProvideBaseUrl()
	{
		var target = Balancer.Next();

		if (target is null)
			throw new NullReferenceException("No remote servers configured.");

		return target.Address;
	}

	private void Initialize()
	{
		var remoteConfiguration = new RemoteConfiguration(Configuration.GetSection("remote"));

		foreach (var s in remoteConfiguration.Servers)
			if (!string.IsNullOrWhiteSpace(s.Address))
				Balancer.Register(new RemoteDescriptor(s.Address));
	}
}
