﻿using System.Collections.Concurrent;

namespace Connected.Net;

internal class RoundRobin
{
	public RoundRobin()
	{
		Items = new();
	}

	private ConcurrentQueue<RemoteDescriptor> Items { get; }

	public void Register(RemoteDescriptor value)
	{
		Items.Enqueue(value);
	}

	public RemoteDescriptor? Next()
	{
		if (Items.IsEmpty)
			return default;

		if (TryFirst() is RemoteDescriptor hit && !string.IsNullOrWhiteSpace(hit.Address))
			return hit;

		if (Items.TryDequeue(out RemoteDescriptor? result))
			Items.Enqueue(result);
		/*
		 * TODO: make this algorithm smarter and use error prone remotes less frequently.
		 */
		return result;
	}

	private RemoteDescriptor? TryFirst()
	{
		if (Items.Count == 1)
		{
			if (Items.TryPeek(out RemoteDescriptor? r))
				return r;
		}

		return default;
	}
}