﻿using Connected.Events;
using Connected.Notifications;
using Connected.ServiceModel;
using Microsoft.Extensions.DependencyInjection;

namespace Connected.Services;

public abstract class EntityService<TPrimaryKey> : IServiceNotifications<TPrimaryKey>, IDisposable
{
	public event ServiceEventHandler<PrimaryKeyEventArgs<TPrimaryKey>>? Inserted;
	public event ServiceEventHandler<PrimaryKeyEventArgs<TPrimaryKey>>? Updated;
	public event ServiceEventHandler<PrimaryKeyEventArgs<TPrimaryKey>>? Deleted;

	protected EntityService()
	{
		if (ClientStartup.Services.GetService<IEventService>() is IEventService events)
			events.Event += OnEvent;
	}

	private async void OnEvent(IOperationState? sender, RemoteEventArgs? e)
	{
		if (!string.Equals(e.Service, GetType().Name, StringComparison.Ordinal))
			return;

		var id = Convert.ChangeType(e.Arguments, typeof(TPrimaryKey));
		var def = typeof(PrimaryKeyEventArgs<>).MakeGenericType(typeof(TPrimaryKey));

		if (Activator.CreateInstance(def, id) is not PrimaryKeyEventArgs<TPrimaryKey> instance)
			return;

		if (string.Equals(e.Event, nameof(Inserted), StringComparison.Ordinal))
			Inserted?.Invoke(sender, instance);
		else if (string.Equals(e.Event, nameof(Updated), StringComparison.Ordinal))
			Updated?.Invoke(sender, instance);
		else if (string.Equals(e.Event, nameof(Deleted), StringComparison.Ordinal))
			Deleted?.Invoke(sender, instance);

		await Task.CompletedTask;
	}

	public void Dispose()
	{
		if (ClientStartup.Services.GetService<IEventService>() is IEventService events)
			events.Event -= OnEvent;
	}
}
